import axios from 'axios'
import { MessageBox, Message } from 'element-ui'
import store from '@/store'
import { getToken } from '@/utils/auth'
import Vue from 'vue'

// create an axios instance
const service = axios.create({
  baseURL: '/api/', // url = base url + request url
  // withCredentials: true, // send cookies when cross-domain requests
  timeout: 5000 // request timeout
})

let loading = {}
function startLoading(el) {
  if (!el) return
  let target = document.querySelector(el)
  if (!target || loading[el]) return
  loading[el] = Vue.prototype.$loading({
    lock: true,
    target
  })
}

function endLoading(el) {
  if (el && loading[el]) {
    loading[el].close()
    delete loading[el]
  }
}

// request interceptor
service.interceptors.request.use(
  config => {
    // do something before request is sent
    startLoading(config.el)
    // config.data = json.dumps(config.data)
    config.headers.get['Content-Type'] = 'application/json'
    if (store.getters.token) {
      config.headers['Authorization'] = getToken()
    } else {
      config.headers['Authorization'] = '121212'
    }
    return config
  },
  error => {
    // do something with request error
    startLoading(error.request.config.el)
    return Promise.reject(error)
  }
)

// response interceptor
let errorShowing = false
service.interceptors.response.use(
  response => {
    const res = response.data.data ? response.data.data : response.data
    let config = response.config
    endLoading(config.el)
    let resFinal = res.rsp || res || {}
    let code = resFinal.errorCode || resFinal.code
    let message = resFinal.message || resFinal.errorMsg || resFinal.error || '接口错误'
    if(code) {
      if (code == 501) {
        location.href = location.href.replace(/#.+/, '') + '#/login'
        return
      }
      if (code == '501') {
        !config.silence && !errorShowing && Message.error(message)
        errorShowing = true
        setTimeout(() => { errorShowing = false }, 3000)
        store.dispatch('user/resetToken')
        return
      }
      // if (code) {
      //   !config.silence && Message.error(message)
      //   return Promise.reject(new Error(message))
      // }
    }
    // if (res.code !== 20000) {
    //   Message({
    //     message: res.message || 'Error',
    //     type: 'error',
    //     duration: 5 * 1000
    //   })

    //   // 50008: Illegal token; 50012: Other clients logged in; 50014: Token expired;
    //   if (res.code === 501 || res.code === 201 || res.code === 50014) {
    //     // to re-login
    //     MessageBox.confirm('You have been logged out, you can cancel to stay on this page, or log in again', 'Confirm logout', {
    //       confirmButtonText: '重新登陆',
    //       cancelButtonText: '取消',
    //       type: 'warning'
    //     }).then(() => {
    //       store.dispatch('user/resetToken').then(() => {
    //         location.reload()
    //       })
    //     })
    //   }
    //   return Promise.reject(new Error(res.message || 'Error'))
    // } else {
      return res
    // }
  },
  error => {
    console.log('err' + error) // for debug
    if (error.response) {
      endLoading(error.response.config.el)
      if (!error.response.config.silence) {
        Message.error(error.message)
      }
    }
    // Message({
    //   message: error.message,
    //   type: 'error',
    //   duration: 5 * 1000
    // })
    return Promise.reject(error)
  }
)

export default service
